// ver 2.0 2008/01/06
// common functions 
// calc_massex 
//   calc mass excess
// calc_mass
//   calc mass from mass excess
// get_atomic_number
//   get atomic nuber from the reaction name
double calc_massex(int A, int Z);
double calc_mass(int A, int Z, double massex);
int get_atomic_number(char *reaction, int *a, int *z);
double bethe(double inE, int Zp, double mass, double thickness, int sw, int target);
double slope(double Es, double mass, int Zp, int target);
double get_ex_toi(char *buf);
int get_parameter(char runname[], double *E, double *FC, int *posi, int *target);

// get the atomic number form the reaction name
int get_atomic_number(char *reaction, int *a, int *z)
{
	FILE *list;
	int readitem, ta[4], tz[4], i;
	char buffer[BUFSIZE], title[BUFSIZE];

	list = fopen("reaction.txt", "r");
	if (list == NULL) {
		return FALSE;
	}

	while (1) {
		if(fgets(buffer,sizeof(buffer),list) == NULL) {
			break;
		}
		readitem = sscanf(buffer, "%s %d %d %d %d %d %d %d %d", 
				title, &ta[1], &tz[1], &ta[0], &tz[0], &ta[2], &tz[2], &ta[3], &tz[3]);

		if (readitem != 9)
			continue;

		// compare n characters from the list
		if (strncmp(title, reaction, strlen(title)) == 0) {
			for (i = 0;i < 4; i++) { 
				a[i] = ta[i];
				z[i] = tz[i];
			}
			fclose(list);
			return TRUE;
		}
	}
	fclose(list);
	printf("%s is not registered reaction.\nplease, define in reaction.txt\n", reaction);
	return FALSE;
}

double calc_massex(int A, int Z)
{
	FILE *data;
	int ta,tz, readitem;
	float massex;
	char buf[BUFSIZE];

	// awm.txt is the list of the mass excess 
	data = fopen("awm.txt", "r");
	if (data == NULL) {
		return 0;
	}

	while (1) {
		if (fgets(buf,sizeof(buf),data) == NULL)
			break;
		readitem = sscanf(buf,"%d %d %f",&ta, &tz, &massex);
		if ((A == ta) && (Z == tz)) {
			fclose(data);
			return (double)massex/(1e3);
		}
	}
	fclose(data);
	return 0;
}

double calc_mass(int A, int Z, double massex)
{
	return ((double)A)*AMU-((double)Z)*ME+massex;
}

// bethe bloch calculation
// Zp projectile charge
// mass projectile mass
// thickness target thickness
// sw = true normal bethe bloch
// sw = false inverse bethe bloch
double bethe(double inE, int Zp, double mass, double thickness, int sw, int target)
{
	double dx, dEdx = 0.; 
	double k1,k2,k3,k4;
	int sign;
	long int i;

	dx = thickness/LOOP;
	// sw == TRUE normal bethe bloch 
	if (sw == TRUE) {
		sign = 1;
	} else {
		sign = -1;
	}

	for(i=0;i<=LOOP;i++){
		k1 = dx*slope(inE,mass,Zp,target);
		k2 = dx*slope(inE+k1/2,mass,Zp,target);
		k3 = dx*slope(inE+k2/2,mass,Zp,target);
		k4 = dx*slope(inE+k3,mass,Zp,target);
		dEdx = (k1 + 2.0e0*k2 + 2.0e0*k3 + k4)/(6.0e0);
		inE = inE + sign*dEdx;
		if (inE <= 0.) break;
	}
					
	return inE; 
}

// bethe bloch calculation for different target
double slope(double Es, double mass, int Zp, int target)
{
	double s,e, FACvalue[3], logIeff;

	if (target == SILICON) { 
		FACvalue[0] = FAC*Z_Si*Zp*Zp/A_Si;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_Si/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == ALMTRGT) {
		FACvalue[0] = FAC*Z_Al*Zp*Zp/A_Al;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_Al/mass) + 2.*FACvalue[0];
		return s;
	}
	
	if (target == C13TRGT) {
		FACvalue[0] = FAC*Z_C13*Zp*Zp/A_C13;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_C13/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == CH2TRGT ) {
		FACvalue[0] = FAC*(Z_C12+2*Z_H)*Zp*Zp/(A_C12+2*A_H);
		logIeff=(12*log(I_C12)+2*log(I_H))/14;
		s  = -FACvalue[0]*mass/Es*(log(4.*ME*Es/mass)-logIeff) + 2.*FACvalue[0];
		return s;
	}
	if (target == MLMNTRGT ) {
		FACvalue[0] = FAC*(6*Z_N+6*Z_H+3*Z_C12)*Zp*Zp/(3*A_C12+6*A_H+6*A_N);
		logIeff=(14*6*log(I_N)+12*3*log(I_C12)+3*log(I_H))/(3*A_C12+6*A_H+6*A_N);
		s  = -FACvalue[0]*mass/Es*(log(4.*ME*Es/mass)-logIeff) + 2.*FACvalue[0];
		return s;
	}
	/*
	if (target == AU) {

		if(Zp == 2){
			e=Es*3727.379/mass;
			if(Es >= 1.0) s  = -(377.571/e*log( 2.50231*e) + 42.2239);
			if(Es < 1.0 && Es > 0.3 ) s  =  -(108.751 +  841.99 *e -832.19*e*e+  271.275 *e*e*e);
			return s;
		}
		
		if(Zp == 1){
			//FACvalue[0] = FAC*Z_Au*Zp*Zp/A_Au;
			//s  = -FACvalue[0]*mass/Es*log(4.*ME*Es/I_Au/mass) + 2.*FACvalue[0];
			e=Es*938.272/mass;
			if(Es >= 0.2) s  = -( 20.5895 /e*log(12.8578*e) + 9.4756);
			//if(Es < 1.0 && Es > 0.3 ) s  =  -(108.751 +  841.99 *Es -832.19*Es*Es+  271.275 *Es*Es*Es);
			return s;
		}
		
	}*/	
	return 0;
}
/*
double slope(double Es, double mass, int Zp, int target)
{
	double s,s2,s3, FACvalue[3], returnvalue;

	if (target == SILICON) { 
		FACvalue[0] = FAC*rho_Si*Z_Si*Zp*Zp/A_Si;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_Si/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == ALMTRGT) {
		FACvalue[0] = FAC*rho_Al*Z_Al*Zp*Zp/A_Al;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_Al/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == CH2TRGT) {
		FACvalue[0] = FAC*rho_C12atCH2*Z_C12*Zp*Zp/A_C12;
		FACvalue[1] = FAC*rho_HatCH2*Z_H*Zp*Zp/A_H;
		s  = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_C12/mass) + 2.*FACvalue[0];
		s2 = - FACvalue[1]*mass/Es*log(4.*ME*Es/I_H/mass) + 2.*FACvalue[1]; 
		returnvalue = rho_CH2*w_CatCH2/rho_C12atCH2*s + rho_CH2*w_HatCH2/rho_HatCH2*s2;
		return returnvalue;
	}
	if (target == C13TRGT) {
		FACvalue[0] = FAC*rho_C13*Z_C13*Zp*Zp/A_C13;
		s = - FACvalue[0]*mass/Es*log(4.*ME*Es/I_C13/mass) + 2.*FACvalue[0];
		return s;
	}
	if (target == MLMNTRGT) {
		FACvalue[0] = FAC*rho_C12atMERANIN*Z_C12*Zp*Zp/A_C12;
		FACvalue[1] = FAC*rho_HatMERANIN*Z_H*Zp*Zp/A_H;
		FACvalue[2] = FAC*rho_NatMERANIN*Z_N*Zp*Zp/A_N;
		s  = -FACvalue[0]*mass/Es*log(4.*ME*Es/I_C12/mass) + 2.*FACvalue[0];
		s2 = -FACvalue[1]*mass/Es*log(4.*ME*Es/I_H/mass) + 2.*FACvalue[1];
		s3 = -FACvalue[2]*mass/Es*log(4.*ME*Es/I_N/mass) + 2.*FACvalue[2];
		returnvalue = rho_MERANIN*w_CatMERANIN/rho_C12atMERANIN*s
			+ rho_MERANIN*w_HatMERANIN/rho_HatMERANIN*s2
			+ rho_MERANIN*w_NatMERANIN/rho_NatMERANIN*s3;

		return returnvalue;
	}
	return 0;
}
*/
// get excited energy from the table of isotopes file
double get_ex_toi(char *title)
{
	FILE *toifile;
	char buffer[BUFSIZE], reactionname[BUFSIZE];
	double Ex, returnE;
	int readitem;

	toifile = fopen("toi.txt", "r");
	if (toifile == NULL) return 0;
	returnE = 0;
	while (1) {
		if (fgets(buffer,sizeof(buffer),toifile) == NULL) 
			break;
		readitem = sscanf(buffer,"%s %lf", reactionname, &Ex);
		if (readitem != 2)
			continue;
		if (strcmp(title, reactionname) == 0) {
			returnE = Ex;
			return returnE;
		}
	}
	printf("ALERT:%s not registered, in toi.txt\n", title);
	fclose(toifile);
	return returnE;
}

// get parameter from rundatalist.txt (position target Energy, and reduced FC-value)
int get_parameter(char runname[], double *E, double *FC, int *posi, int *target)
{
	FILE *datafile;
	char buffer[BUFSIZE], trun[10], ttarget[10],tbeam[10], ttime[10], trunname[] = "run";
	double tladder, tdeg, tdeltaE, tE, tcoin, tdaq, tfc, tclock,tenergy; 
	int readitem;

	// if runname = 0234, need add run0234
	if (strlen(runname) == 4) {
		strcat(trunname,runname);
		strcpy(runname, trunname);
	}

		
	datafile = fopen("rundatalist-plus.txt", "r");
	if (datafile == NULL) return FALSE;
	while (1) {
		if (fgets(buffer,sizeof(buffer),datafile) == NULL) 
			break;
			// target ladder deg time run deltaE E coin daq fc clock beam energy
		readitem = sscanf(buffer, "%s %lf %lf %s %s %lf %lf %lf %lf %lf %lf %s %lf",
			ttarget, &tladder, &tdeg, ttime, trun, &tdeltaE, &tE, &tcoin, &tdaq, &tfc, &tclock,
			tbeam, &tenergy);
		if (readitem != 13)
			continue;

		if (strncmp(trun, runname, strlen(runname)) == 0) {
			*FC = tfc*(tdaq/tE); // fc value
			*E = tenergy;// beam energy
			// target
			if (strcmp(ttarget, "CH2") == 0) *target = CH2TRGT;
			else if (strcmp(ttarget, "Al") == 0) *target = ALMTRGT;
			else if (strcmp(ttarget, "melamine") == 0) *target = MLMNTRGT;
			else if (strcmp(ttarget, "13C") == 0) *target = C13TRGT;
			// position
			if (fabs(tladder - 3.05e2) < 1.0e0) *posi = LADDER_POSI1;
			else if (fabs(tladder - 2.60e2) < 1.0e0) *posi = LADDER_POSI2;
			else if (fabs(tladder - 3.50e2) < 1.0e0) *posi = LADDER_POSI3;

			fclose(datafile);
			return TRUE;
		}
	}
	fclose(datafile);
	printf("run %s, cannot get parameter, in rundatalist.txt\n", runname);
	return FALSE;
}
