// define
// ver 0.9 2007/12/20(Thu)
// ver 1.0 2008/01/06(Fri)
// ver 4.0 2008/02/03(Sun)
// const
#define PI 3.14159265
#define ME 0.510998918 // electron mass
#define AMU 931.494028 // atomic mass unit
#define ELECHARGE 1.60217733e-19 // electron's charge
#define NA 6.02252e23 // Avogadro Number 
#define BUFSIZE 100
#define TRUE 1
#define FALSE 0
#define ERR -1

// cross section calculation
#define COULPRPLSE 1e-10 // Coulomb per Pulse 
#define DEGMAX 180
#define LINEMAX 60
// dOMEGA
#define GEOMETRY1_MIN 139
#define GEOMETRY1_MAX 225
#define dOMEGA_1 8.49850852e-4
#define GEOMETRY2_MIN 238
#define GEOMETRY2_MAX 380
#define dOMEGA_2 7.2121e-4

// excited energy calculation
#define DATAMAX 50
#define MAXTRIAL 10 // number of mesh in the target

// energy calibration
#define CALIB1_MIN 139
#define CALIB1_MAX 225
 
#define MEVPERCH1A 4.31893e-3 //4.23326e-3
#define MEVSHIFT1A -0.223807 //1.62225e-1
#define MEVPERCH1HE 4.31893e-3 //4.23326e-3
#define MEVSHIFT1HE -0.223807 //1.62225e-1
#define MEVPERCH1T 4.58343e-3 //4.23326e-3
#define MEVSHIFT1T -0.219162 //1.62225e-1
#define MEVPERCH1D 4.58343e-3 //4.23326e-3
#define MEVSHIFT1D -0.219162 //1.62225e-1
#define MEVPER2CH1D -9.92898e-8 //4.23326e-3
#define MEVPERCH1P 4.42028e-3 //4.23326e-3
#define MEVSHIFT1P -0.180336 //1.62225e-1
#define MEVPER2CH1P -1.89868e-08 //4.23326e-3

#define MEVPERCH1 4.31815e-3 //4.23326e-3
#define MEVSHIFT1 -0.103511 //1.62225e-1

#define CALIB2_MIN 238
#define CALIB2_MAX 299
#define MEVPERCH2A 4.31815e-3 //4.23326e-3
#define MEVSHIFT2A -0.103511 //1.62225e-1
#define MEVPERCH2HE 4.31815e-3 //4.23326e-3
#define MEVSHIFT2HE -0.103511 //1.62225e-1
#define MEVPERCH2T 4.31815e-3 //4.23326e-3
#define MEVSHIFT2T -0.103511 //1.62225e-1
#define MEVPERCH2D 4.31815e-3 //4.23326e-3
#define MEVSHIFT2D -0.103511 //1.62225e-1
#define MEVPERCH2P 4.31815e-3 //4.23326e-3
#define MEVSHIFT2P -0.103511 //1.62225e-1

#define MEVPERCH2 7.6630e-3
#define MEVSHIFT2 1.052e-1

#define CALIB3_MIN 300
#define CALIB3_MAX 380
#define MEVPERCH3A 7.79387e-3
#define MEVSHIFT3A -6.52508e-1
#define MEVPERCH3HE 7.79387e-3
#define MEVSHIFT3HE -6.52508e-1
#define MEVPERCH3T 7.76914e-3
#define MEVSHIFT3T -0.473585 
#define MEVPERCH3D 7.76914e-3 //4.23326e-3
#define MEVSHIFT3D -0.473585 //1.62225e-1
#define MEVPERCH3P 7.76914e-3
#define MEVSHIFT3P -0.473585

#define MEVPERCH3 7.82217e-3
#define MEVSHIFT3 -6.6148e-1

// target setting
#define LADDER_POSI1 0
#define LADDER_POSI2 1
#define LADDER_POSI3 2
//#define targetTHICKNESS 20.0e-4 // 20 micro meter
#define TRGTANGL 45
#define BEFORE 0
#define AFTER 1

// bethe bloch calculation 
#define FAC 0.1535 // MeV cm^2 /g 2pi Na r_e^2 m_e c^2 
#define LOOP  5e2 // loop at runge-kutta 
#define dETHICKNESS 28.9e-4 // 25 micro meter
#define DEADTHICKNESS 0 // depth of dead zone of E counter

// density  g/cm^2
#define rho_Si 2.33 
#define rho_Al 2.69890
#define rho_H  8.37480e-5
#define rho_HatCH2 1.3508834e-1
#define rho_HatMERANIN 0.4012e-3
#define rho_N  1.16528e-3
#define rho_NatMERANIN 5.5756e-3 
#define rho_C12 1.7
#define rho_C12atCH2  8.0491166e-1
#define rho_C12atMERANIN 2.3884e-3
#define rho_C13 2.0 // uncertain
#define rho_O 1.33151e-3
#define rho_CH2 9.4e-1
#define rho_MERANIN 8.3653e-3

// mean excited energy 
#define I_Si 1.73e-4    
#define I_Ge 3.43e-4    
#define I_Al 1.66e-4
#define I_C12 7.8e-5
#define I_C13 7.8e-5
#define I_N 8.2e-5
#define I_H 1.92e-5
#define I_O 9.5e-5
// atomic number 
#define Z_Alpha 2
#define Z_Si 14.
#define Z_Al 13.
#define Z_H 1.
#define Z_Ge 32.
#define Z_C12 6.
#define Z_C13 6.
#define Z_N 7.
//atomic weight
#define A_Alpha 4
#define A_Si 28.1
#define A_Ge 72.6
#define A_C12 12.
#define A_C13 13.0033548378
#define A_N 14.0067
#define A_H 1.00794
#define A_O 15.9994
#define A_Al 26.981538
//fraction of mixture by weight
#define w_CatCH2 0.856289
#define w_HatCH2 0.143711
#define w_CatMERANIN 0.28551  
#define w_HatMERANIN 0.047963 
#define w_NatMERANIN 0.66652

// target list 
#define SILICON 0
#define CH2TRGT 1
#define ALMTRGT 2
#define MLMNTRGT 3
#define C13TRGT 4
// target nuclei
#define SAME 0
#define CNCL 1
#define HNCL 2
#define NNCL 3

// target thickness
#define CH2THICKNESS 17.689967e-4 // 20e-4
#define ALMTHICKNESS 14.7949e-4 // 15e-4
#define MLMTHICKNESS 1.18e-1 // 0.118 cm 
#define C13THICKNESS C13SRFCDNS/rho_C13

// surface density
#define CH2SRFCDNS 1.662867e-3//rho_CH2*CH2THICKNESS
#define ALMSRFCDNS 3.9930e-3//rho_Al*ALMTHICKNESS 
#define MLMSRFCDNS 0.987e-3//rho_MERANIN*MLMTHICKNESS
#define C13SRFCDNS 3.0e-5 // 30 micro gram /cm^2

// alpha energy 241Am
#define ALPHA_E 5.480392188

// for program kinematics plus
// theta list number
#define ALNUM 27
#define C13NUM 13
#define C12NUM 12
#define O16NUM 16
#define H1NUM 1

// theta list
/*
double theta_list_Al[] = { 35.9, 40.9, 45.9, 49.1,50.9, 55.9, 60.9, 65.9, 70.9, 75.9, 80.9, 85.9,
	90.9,100.9,110.9,120.9,130.9,140.9 };
double theta_list_C13[] = { 25.9, 30.9, 35.9, 40.9, 41.8, 45.9, 46.5, 50.9, 55.9, 60.9, 65.9, 70.9, 75.9, 80.9,
	85.9, 90.9 };
double theta_list_O16[] = { 25.9, 30.9, 35.9, 40.9, 41.8, 45.9, 46.5, 50.9, 55.9, 60.9, 65.9, 70.9, 75.9, 80.9,
	85.9, 90.9 };
double theta_list_C12[] = { 30.9, 35.9, 40.9, 45.9, 50.9, 55.9, 60.9, 65.9, 70.9, 75.9, 80.9, 85.9, 90.9 };
double theta_list_H1[] = { 30.9, 35.9, 40.9, 45.9, 50.9, 55.9, 60.9, 65.9, 70.9, 75.9, 80.9, 85.9, 90.9 };
*/
double theta_list_Al[] = 
	{ 20.9, 25.9 , 30.9, 35.9, 40.9, 45.9, 50.9, 55.9, 60.9};
double theta_list_CH2[] = 
	{ 18.9, 20.9, 23.9, 25.9, 28.9, 
		30.9, 33.9, 35.9, 38.9, 40.9, 43.9, 45.9, 48.9,
		50.9, 53.9, 55.9, 58.9, 60.9, 63.9, 65.9, 68.9, 70.9, 
		73.9, 75.9, 78.9, 80.9, 83.9, 85.9, 88.9, 90.9, 100.9, 110.9, 120.9 };
double theta_list_C13[] = 
	{ 18.9, 20.9, 23.9, 25.9, 28.9, 30.9, 33.9,
	35.9, 38.9, 40.9, 43.9, 45.9, 48.9, 50.9, 55.9, 53.8,60.9, 65.9, 70.9, 
	75.9, 80.9, 90.9 ,120.9 };
double theta_list_Melamine[] = 
	{ 20.9, 25.9, 30.9, 35.9, 40.9, 45.9, 50.9, 55.9, 60.9, 70.9, 80.9};
// theta define
#define THETAMAX 180
#define DELTATHETA 5
