// excited energy calculation
// ver 1.0 2007/12/18
// ver 2.0 2007/12/20
// ver 3.0 
// ver 4.0 2008/01/06
// ver 5.0 2008/02/08

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "defines.h"
#include "commonfunction.h"

void do_calc(char *buf);
double calc_excited_energy(double *mass, double Q, double E1, double E3, double theta); // calc excited energy
void do_summary(char *buf);
int check_summary(char *buf);
double get_pathlength(double theta, int i, int posi, int target, int sw); // get pathlength
double calc_energy(double *mass, double Q, double E, double Ex, double theta, int sw); // calc energy after scttering
double Ch2MeV(double channel, char *runname,int z,int a); // Channel -> Energy 

int main(int argc, char *argv[])
{
  FILE *fp;
  char buf[BUFSIZE];

  // parameter file open 
  fp = fopen(argv[1], "r");
  if(fp == NULL) {
	printf("cannot open file(1)\n");
    return;
  }
  
  // calc excited energy 
  while (1) {
	 if (fgets(buf,sizeof(buf),fp) == NULL)
		  break;

	 do_calc(buf);
  }
  fclose(fp);

  // summary 
  fp = fopen(argv[1], "r");
  if (fp == NULL) {
	  printf("cannot open file(2)\n");
	  return;
  }
  while (1) {
	  if (fgets(buf,sizeof(buf),fp)==NULL)
		  break;
	  do_summary(buf);
  }
  fclose(fp);
}

void do_calc(char *buf)
{
	char title[BUFSIZE],runname[BUFSIZE],targettitle[BUFSIZE];
	int readitem, i,j, a[4], z[4], sw, posi, target;
	double mass[4], massex[4], Q, deg, channel, error, E, FC;
	double Ecount, Etotal, Ex, pathlength_before, pathlength_after, ExfromTOI;
	double Etotal_trial,Etotal_trial_series[MAXTRIAL+1],Ex_trial, Ex_min, Ex_max;
	double Ejustbefore,Ejustafter;
	FILE *fileout;

	// read parameters 
	readitem = sscanf(buf, "%s %s %lf %lf %lf",
			runname, title, &deg, &channel, &error);

	 if (readitem != 5) 
		 return;
	 // get other parameters, position, energy, and target
	 if (!get_parameter(runname, &E, &FC, &posi, &target)) {
		 return;
	 }
	 // open output file sw == 1 means need new file 
	 sw = check_summary(title);
	 if (sw)
		 fileout = fopen(title, "w");
	 else
		 fileout = fopen(title, "a");

	 if (fileout == NULL)
		 return;

	 // get atomic number
	 if (!get_atomic_number(title,a,z)) {
		 printf("%s not registered reaction.\n", title);
		 return;
	 }

	 // calc mass 
	 for (i = 0; i < 4; i++) {
		 massex[i] = calc_massex(a[i], z[i]);
		 mass[i] = calc_mass(a[i],z[i],massex[i]);
	 }

	 if (sw == 1) { printf("\n"); }
	 printf("calculating %s %s %3.1lf deg\n", title, runname, deg); // monitor
	 
	 Q = massex[0]+massex[1]-massex[2]-massex[3];

	 // calculation energy 
	 Ecount = Ch2MeV(channel,runname,z[2],a[2]); // channel -> energy (in E-counter)
	 Etotal = bethe(Ecount,z[2],mass[2],dETHICKNESS*rho_Si,FALSE,SILICON);
	 Ex_trial = calc_excited_energy(mass,Q,E,Etotal,deg); // no energy loss case
	 printf("Ex=%lf\n", Ex_trial); // monitor

	 Ex_min = Ex_trial - 1.4; // minimum
	 Ex_max = Ex_trial+0.2; // maximum
	 Ex_trial = Ex_trial-0.4;
	
	i=0;
	while(1){
		 // calc energy after, in trial Excited energy
		 for (j = 0; j <= MAXTRIAL; j++) {
			 pathlength_before = get_pathlength(deg,j,posi,target,BEFORE);
			 pathlength_after = get_pathlength(deg,j,posi,target,AFTER);
			 if ((pathlength_after == ERR) || (pathlength_before == ERR)) continue; 
			 			// if failed get pathlength
			 Ejustbefore = bethe(E,z[0],mass[0],pathlength_before,TRUE,target); 
			 			// calc energy loss before scattering
			 Ejustafter = calc_energy(mass,Q,Ejustbefore,Ex_trial,deg,TRUE); 
			 			// calc scattering in trial Excited energy
			 Etotal_trial_series[j] = bethe(Ejustafter,z[2],mass[2],pathlength_after,TRUE,target); 
			 			// calc energy loss after scattering
		 }
		 // get the mean of Etotal trial
		 Etotal_trial = 0;
		 for (j = 0; j <= MAXTRIAL; j++) {
			 Etotal_trial += Etotal_trial_series[j];
		 }
		 Etotal_trial = Etotal_trial/(MAXTRIAL+1);
		 // check Etotal_trial
		 if (Etotal_trial > Etotal) { // Ex_trial < Ex_real
			 Ex_min = Ex_trial;
			 Ex_trial = (Ex_trial + Ex_max)/2.0;
		 }
		 if (Etotal_trial < Etotal) { // Ex_trial > Ex_real
			 Ex_max = Ex_trial;
			 Ex_trial = (Ex_trial + Ex_min)/2.0;
		 }
		 if (fabs(Ex_max - Ex_min) < 1.0e-4) {
			 break; // no need more calculation
		 }
		if(i > 15){
			printf("Ex is not reasonable\n");
			break;
		}
		
         i++;
	}
	 Ex = Ex_trial;
	 //printf("Ex=%lf, Etotal%lf, Etotaltrial=%lf\n", Ex, Etotal, Etotal_trial); // monitor

	 if (sw == 1) { // make new output file 
		 // template 
		 // target name
		 if (target == CH2TRGT) {  strcpy(targettitle, "CH2"); }
		 else if (target == C13TRGT) { strcpy(targettitle, "13C"); }
		 else if (target == MLMNTRGT) { strcpy(targettitle, "Melamine"); }
		 else if (target == ALMTRGT) { strcpy(targettitle, "Aluminum"); }
		 else { strcpy(targettitle, "UNKNOWN");  }

		 // get excited energy (from the table of isotope) .. 
		 ExfromTOI = get_ex_toi(title);
		 fprintf(fileout, "#%s in %s, %4.3lf from table of isotopes\n", title, targettitle, ExfromTOI);
         	 fprintf(fileout, "#Q-value %3.2lf, E = %3.2lf\n", Q, E);
		 fprintf(fileout, "#run, deg, E-dE(ch), err, (MeV), total, Ex\n");
	 }

	 fprintf(fileout, "%s %5.1lf %7.1lf %4.2lf %5.4lf %5.4lf %5.4lf\n",
			 runname, deg, channel, error, Ecount, Etotal, Ex);

	 fclose(fileout);
}

double calc_excited_energy(double *mass, double Q, double E1, double E3, double theta)
{
	// calc excited energy 
	double mass_ratio[2];
	double Ex;

	mass_ratio[0] = mass[0]/mass[3];
	mass_ratio[1] = mass[2]/mass[3];

	Ex = 2.0e0*sqrt(mass_ratio[0]*E1*mass_ratio[1]*E3)*cos(theta*PI/180)
		- (1 + mass_ratio[1])*E3 + (1 - mass_ratio[0])*E1 + Q;
	return Ex;
}

void do_summary(char *buf)
{
	FILE *fileout;
	FILE *texout;
	char title[BUFSIZE], runname[BUFSIZE], buffer[BUFSIZE];
	double deg, channel, error, Ecount, Etot, Ex, ExToi; 
	double ExData[DATAMAX], ExMean = 0, sigma = 0;
	int readitem,datanum = 0, i=0;

	sscanf(buf, "%s %s", runname, title);

	if (check_summary(title))
		return;

	fileout = fopen(title, "r");
	if (fileout == NULL) {
		return;
	}

	// read the parameter
	while (1) {
		if(fgets(buffer,sizeof(buffer),fileout) == NULL)
			break;
		readitem = sscanf(buffer, "%s %lf %lf %lf %lf %lf %lf",
				runname, &deg, &channel, &error, &Ecount, &Etot, &Ex);
		if (readitem != 7) {
			continue;
		}
		ExData[datanum] = Ex;
		datanum++;
	}
	fclose(fileout);
	if (datanum == 0)
		return;

	fileout = fopen(title, "a");
	if (fileout == NULL) {
		return;
	}
	// calc the summary 
	for (i = 0; i < datanum; i++) {
		ExMean += ExData[i];
	}

	ExMean = ExMean / datanum;

	for (i = 0; i < datanum; i++) {
		sigma += (ExData[i]-ExMean)*(ExData[i]-ExMean);
	}

	if (datanum != 1) {
		sigma = sqrt(sigma/(datanum-1));
	} else {
		sigma = 0;
	}

	// out put 1
	ExToi = get_ex_toi(title);	
	fprintf(fileout, "#summary\n");
	fprintf(fileout, "#Ex = %4.3lf from table of isotpes\n", ExToi);
	fprintf(fileout, "#data number %d\n", datanum);
	fprintf(fileout, "#Excited energy mean %4.3lf\n", ExMean);
	fprintf(fileout, "#Standard deviation  %4.3lf\n", sigma);
	fclose(fileout);

	// monitor
	printf("%s, %4.3lf from table of isotopes\n", title, ExToi);
	printf("data %2d, excited energy %4.3lf plusminus %4.3lf\n", datanum, ExMean, sigma);

	// output 2
	texout = fopen("output.txt", "a");
	fprintf(texout, "#summary %s, Ex = %4.3lf from table of isotopes\n", title, ExToi);
	fprintf(texout, "#data number %2d\n", datanum);
	fprintf(texout, "#Excited energy mean %4.3lf\n", ExMean);
	fprintf(texout, "#Standard deviation  %4.3lf\n\n", sigma);
	fclose(texout);
}

int check_summary(char *buf)
{
	// return true if there "#summary" strings 
	FILE *fp;
	char buffer[BUFSIZE], runname[BUFSIZE], outputfile[BUFSIZE];

	sscanf(buf, "%s", outputfile);
	fp = fopen(outputfile, "r");
	if (fp == NULL) {
		return 1;
	}

	while (1) {
		if (fgets(buffer,sizeof(buffer),fp) == NULL)
			break;

		sscanf(buffer, "%s", runname);
		if (strcmp(runname, "#summary") == 0) {
			fclose(fp);
			return 1;
		}
	}
	return 0;
}

double get_pathlength(double theta, int i, int posi, int target, int sw)
{
	double thickness, pathlength;

		// get thickness
	if (target == CH2TRGT) {
		thickness = CH2SRFCDNS/*CH2THICKNESS*/;
	} else if (target == ALMTRGT) {
		thickness = ALMSRFCDNS/*ALMTHICKNESS*/;
	} else if (target == MLMNTRGT) {
		thickness = MLMSRFCDNS/*MLMTHICKNESS*/;
	} else if (target == C13TRGT) {
		thickness = C13SRFCDNS/*C13THICKNESS*/;
	} else {
		return ERR;
	}

	// before
	if (sw == BEFORE) {
		if (posi == LADDER_POSI1) {
			pathlength = thickness*((double)i/(double)MAXTRIAL);
		} else if (posi == LADDER_POSI2) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)*(1/cos(TRGTANGL*PI/180));
		} else if (posi == LADDER_POSI3) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)*(1/cos(TRGTANGL*PI/180));
		} else {
			pathlength = ERR;
		}
	}

	// after
	if (sw == AFTER) {
		if (posi == LADDER_POSI1) {
			pathlength = thickness*(1-(double)i/(double)MAXTRIAL)*(1/cos(theta*PI/180));
		} else if (posi == LADDER_POSI2) {
			pathlength = thickness*(1 - ((double)i/(double)MAXTRIAL)/(sqrt(2)))/cos((theta-TRGTANGL)*PI/180);
		} else if (posi == LADDER_POSI3) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)/(fabs(cos((theta+TRGTANGL)*PI/180)));
		} else {
			return ERR;
		}
	}
	return pathlength;
}

double calc_energy(double *mass, double Q, double E, double Ex, double theta, int sw)
{
	double A,B,C,v,Ep,Ey;

	theta = theta*PI/1.8e2;

// calc energy in Newton Dynamics 
	A = mass[2]*(mass[2]+mass[3]);
	B = sqrt(2.0e0*mass[0]*E)*mass[2]*cos(theta);
	C = 2.0e0*mass[0]*E - 2.0e0*mass[3]*(E+Q-Ex);
	 
	if (B*B - A*C < 0) return 0;

	if (sw == TRUE) {
		v = (B+sqrt(B*B-A*C))/A;
	} else if (sw == FALSE) {
		v = (B-sqrt(B*B-A*C))/A;
	}

	Ep = 5.0e-1*mass[2]*v*v;

	// check energy conservation 
	Ey = (1/(2.0e0*mass[3]))*(2.0e0*mass[0]*E+2.0e0*mass[2]*Ep)
		-(sqrt(2.0e0*mass[0]*E*2.0e0*mass[2]*Ep)/(mass[3]))*cos(theta)
		+Ex;

	if (fabs((E+Q)-(Ey+Ep)) < 1e-3) {
		return Ep;
	} else { 
		return 0;
	}
}

double Ch2MeV(double channel, char *runname,int z,int a)
{
	int runnumber=0,i;
	double energy;
	// runname(run0235) -> runnumber(235) transformation
	for (i = 0; i < strlen(runname); i++) {
		if (runname[i] == 'r' || runname[i] == 'u' 
				|| runname[i] == 'n' || runname[i] == '0')
			continue;
		break;
	}	
	runnumber = atoi(&(runname[i]));
	if (runnumber >= CALIB1_MIN && runnumber <= CALIB1_MAX) {
		
		if(z==2&&a==4) energy = MEVPERCH1A*channel+MEVSHIFT1A;
		else if(z==2&&a==3) energy = MEVPERCH1HE*channel+MEVSHIFT1HE;
		else if(z==1&&a==3) energy = MEVPERCH1T*channel+MEVSHIFT1T;
		else if(z==1&&a==2) energy = MEVPERCH1D*channel+MEVSHIFT1D+MEVPER2CH1D*channel*channel;
		else if(z==1&&a==1) energy = MEVPERCH1P*channel+MEVSHIFT1P+MEVPER2CH1P*channel*channel;
		else energy = 0;
		
		return energy;
	} else if (runnumber >= CALIB2_MIN && runnumber <= CALIB2_MAX) {
		
		if(z==2&&a==4) energy = MEVPERCH2A*channel+MEVSHIFT2A;
		else if(z==2&&a==3) energy = MEVPERCH2HE*channel+MEVSHIFT2HE;
		else if(z==1&&a==3) energy = MEVPERCH2T*channel+MEVSHIFT2T;
		else if(z==1&&a==2) energy = MEVPERCH2D*channel+MEVSHIFT2D;
		else if(z==1&&a==1) energy = MEVPERCH2P*channel+MEVSHIFT2P;
		else energy = 0;
		
		return energy;
	} else if (runnumber >= CALIB3_MIN && runnumber <= CALIB3_MAX) {
		
		if(z==2&&a==4) energy = MEVPERCH3A*channel+MEVSHIFT3A;
		else if(z==2&&a==3) energy = MEVPERCH3HE*channel+MEVSHIFT3HE;
		else if(z==1&&a==3) energy = MEVPERCH3T*channel+MEVSHIFT3T;
		else if(z==1&&a==2) energy = MEVPERCH3D*channel+MEVSHIFT3D;
		else if(z==1&&a==1) energy = MEVPERCH3P*channel+MEVSHIFT3P;
		else energy = 0;
		
		return energy;
	} else {
		printf("ERROR:NOT FOUND run %d 's calibration parameter\n",runnumber);
		return 0;
	}
}
