// excited energy calculation mini-version

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "defines.h"
#include "commonfunction.h"

void do_calc(char *,char *, char *, char *);
double calc_excited_energy(double *mass, double Q, double E1, double E3, double theta); // calc excited energy
double get_pathlength(double theta, int i, int posi, int target, int sw); // get pathlength
double calc_energy(double *mass, double Q, double E, double Ex, double theta, int sw); // calc energy after scttering
double Ch2MeV(double channel, char *runname); // Channel -> Energy 

int main(int argc, char *argv[])
{
  char buf[BUFSIZE];

  if (argc != 5) {
  	printf("please input--example\n   run0235 13C3Hed14N1st 18.9 1800.5\n");
	return 0;
  }
  // input ./exmini runname degrees reaction channel
  do_calc(argv[1], argv[2], argv[3], argv[4]);
}

void do_calc(char *buf_1, char *buf_2, char *buf_3, char *buf_4)
{
	char title[BUFSIZE],runname[BUFSIZE],targettitle[BUFSIZE];
	int readitem, i,j, a[4], z[4], sw, posi, target;
	double mass[4], massex[4], Q, deg, channel, error, E, FC;
	double Ecount, Etotal, Ex, pathlength_before, pathlength_after, ExfromTOI;
	double Etotal_trial,Etotal_trial_series[MAXTRIAL+1],Ex_trial, Ex_min, Ex_max;
	double Ejustbefore,Ejustafter;

	// read parameters 
	readitem = 0;
	readitem += sscanf(buf_1, "%s", runname);
	readitem += sscanf(buf_2, "%s", title);
	readitem += sscanf(buf_3, "%lf", &deg);
	readitem += sscanf(buf_4, "%lf", &channel);

	 if (readitem != 4)  {
		 return;
	 }
	 // get other parameters, position, energy, and target
	 if (!get_parameter(runname, &E, &FC, &posi, &target)) {
		 return;
	 }
	 // get atomic number
	 if (!get_atomic_number(title,a,z)) {
		 printf("%s not registered reaction.\n", title);
		 return;
	 }

	 // calc mass 
	 for (i = 0; i < 4; i++) {
		 massex[i] = calc_massex(a[i], z[i]);
		 mass[i] = calc_mass(a[i],z[i],massex[i]);
	 }

	 Q = massex[0]+massex[1]-massex[2]-massex[3];

	 // calculation energy 
	 Ecount = Ch2MeV(channel,runname); // channel -> energy (in E-counter)
	 Etotal = bethe(Ecount,z[2],mass[2],dETHICKNESS+DEADTHICKNESS,FALSE,SILICON);
	 Ex_trial = calc_excited_energy(mass,Q,E,Etotal,deg); // no energy loss case
	 Ex_min = Ex_trial - 1.0e0; // minimum
	 Ex_max = Ex_trial + 1.0e0; // maximum

	 for (i = 0; i <= MAXTRIAL; i++) {
		 // calc energy after, in trial Excited energy
		 for (j = 0; j <= MAXTRIAL; j++) {
			 pathlength_before = get_pathlength(deg,j,posi,target,BEFORE);
			 pathlength_after = get_pathlength(deg,j,posi,target,AFTER);
			 if ((pathlength_after == ERR) || (pathlength_before == ERR)) continue; 
			 			// if failed get pathlength
			 Ejustbefore = bethe(E,z[0],mass[0],pathlength_before,TRUE,target); 
			 			// calc energy loss before scattering
			 Ejustafter = calc_energy(mass,Q,Ejustbefore,Ex_trial,deg,TRUE); 
			 			// calc scattering in trial Excited energy
			 Etotal_trial_series[j] = bethe(Ejustafter,z[2],mass[2],pathlength_after,TRUE,target); 
			 			// calc energy loss after scattering
		 }
		 // get the mean of Etotal trial
		 Etotal_trial = 0;
		 for (j = 0; j <= MAXTRIAL; j++) {
			 Etotal_trial += Etotal_trial_series[j];
		 }
		 Etotal_trial = Etotal_trial/(MAXTRIAL+1);
		 // check Etotal_trial
		 if (Etotal_trial > Etotal) { // Ex_trial < Ex_real
			 Ex_min = Ex_trial;
			 Ex_trial = (Ex_trial + Ex_max)/2.0;
		 }
		 if (Etotal_trial < Etotal) { // Ex_trial > Ex_real
			 Ex_max = Ex_trial;
			 Ex_trial = (Ex_trial + Ex_min)/2.0;
		 }
		 if (fabs(Ex_max - Ex_min) < 1.0e-6) {
			 break; // no need more calculation
		 }
         }
	 Ex = Ex_trial;

	 // get excited energy (from the table of isotope) .. 
	 ExfromTOI = get_ex_toi(title);
	 printf("#%s               %4.3lf from table of isotopes\n", title, ExfromTOI);
	 printf("#run, deg, E-dE(ch), (MeV), Ex\n");
	 printf("%s %5.1lf %7.1lf %5.4lf %5.4lf\n",
			 runname, deg, channel, Ecount, Ex);
}

double calc_excited_energy(double *mass, double Q, double E1, double E3, double theta)
{
	// calc excited energy 
	double mass_ratio[2];
	double Ex;

	mass_ratio[0] = mass[0]/mass[3];
	mass_ratio[1] = mass[2]/mass[3];

	Ex = 2.0e0*sqrt(mass_ratio[0]*E1*mass_ratio[1]*E3)*cos(theta*PI/180)
		- (1 + mass_ratio[1])*E3 + (1 - mass_ratio[0])*E1 + Q;
	return Ex;
}

double get_pathlength(double theta, int i, int posi, int target, int sw)
{
	double thickness, pathlength;

	// get thickness
	if (target == CH2TRGT) { thickness = CH2THICKNESS; }
	else if (target == ALMTRGT) { thickness = ALMTHICKNESS; }
	else if (target == MLMNTRGT) { thickness = MLMNTRGT; }
	else if (target == C13TRGT) { thickness = C13THICKNESS; }
	else { return ERR; }

	// before
	if (sw == BEFORE) {
		if (posi == LADDER_POSI1) {
			pathlength = thickness*((double)i/(double)MAXTRIAL);
		} else if (posi == LADDER_POSI2) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)*(1/cos(TRGTANGL*PI/180));
		} else if (posi == LADDER_POSI3) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)*(1/cos(TRGTANGL*PI/180));
		} else {
			pathlength = ERR;
		}
	}

	// after
	if (sw == AFTER) {
		if (posi == LADDER_POSI1) {
			pathlength = thickness*(1-(double)i/(double)MAXTRIAL)*(1/cos(theta*PI/180));
		} else if (posi == LADDER_POSI2) {
			pathlength = thickness*(1 - ((double)i/(double)MAXTRIAL)/(sqrt(2)))/cos((theta-TRGTANGL)*PI/180);
		} else if (posi == LADDER_POSI3) {
			pathlength = thickness*((double)i/(double)MAXTRIAL)/(fabs(cos((theta+TRGTANGL)*PI/180)));
		} else {
			return ERR;
		}
	}
	return pathlength;
}

double calc_energy(double *mass, double Q, double E, double Ex, double theta, int sw)
{
	double A,B,C,v,Ep,Ey;

	theta = theta*PI/1.8e2;

// calc energy in Newton Dynamics 
	A = mass[2]*(mass[2]+mass[3]);
	B = sqrt(2.0e0*mass[0]*E)*mass[2]*cos(theta);
	C = 2.0e0*mass[0]*E - 2.0e0*mass[3]*(E+Q-Ex);
	 
	if (B*B - A*C < 0) return 0;

	if (sw == TRUE) {
		v = (B+sqrt(B*B-A*C))/A;
	} else if (sw == FALSE) {
		v = (B-sqrt(B*B-A*C))/A;
	}

	Ep = 5.0e-1*mass[2]*v*v;

	// check energy conservation 
	Ey = (1/(2.0e0*mass[3]))*(2.0e0*mass[0]*E+2.0e0*mass[2]*Ep)
		-(sqrt(2.0e0*mass[0]*E*2.0e0*mass[2]*Ep)/(mass[3]))*cos(theta)
		+Ex;

	if (fabs((E+Q)-(Ey+Ep)) < 1e-3) {
		return Ep;
	} else { 
		return 0;
	}
}

double Ch2MeV(double channel, char *runname)
{
	int runnumber=0,i;
	double energy;
	// runname(run0235) -> runnumber(235) transformation
	for (i = 0; i < strlen(runname); i++) {
		if (runname[i] == 'r' || runname[i] == 'u' 
				|| runname[i] == 'n' || runname[i] == '0')
			continue;
		break;
	}	
	runnumber = atoi(&(runname[i]));
	if (runnumber >= CALIB1_MIN && runnumber <= CALIB1_MAX) {
		energy = MEVPERCH1*channel+MEVSHIFT1;
		return energy;
	} else if (runnumber >= CALIB2_MIN && runnumber <= CALIB2_MAX) {
		energy = MEVPERCH2*channel+MEVSHIFT2;
		return energy;
	} else if (runnumber >= CALIB3_MIN && runnumber <= CALIB3_MAX) {
		energy = MEVPERCH3*channel+MEVSHIFT3;
		return energy;
	} else {
		printf("ERROR:NOT FOUND run %d 's calibration parameter\n",runnumber);
		return 0;
	}
}
