// tex -> data
// ver 1.0 2008/01/06
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define TRUE 1
#define FALSE 0
#define BUFSIZE 100
#define ERR -1
// state
#define IN 1
#define OUT 0
// read position
#define HEADER 1
#define HEADEREND 2
#define PARATITLE 3
#define PARA 4

#define HEADERTITLE "\\multicolumn{12}{c}{$\\mathrm{"
#define TOPRULE "\\toprule"
#define MIDRULE "\\midrule"
#define BOTTOMRULE "\\bottomrule"

void do_read(FILE *fp, char title[]);
int get_reaction(char buf[], char reaction[]);
int search_andposi(char buf[], int shift);
void comp_reaction(char buf[], char reaction[]);

main(int argc, char *argv[])
{
	FILE *fp;
	char title[BUFSIZE];

	fp = fopen(argv[1], "r");
	if (fp == NULL) {
		printf("cannot open file(1)\n");
		return;
	}
	
	do_read(fp, argv[1]);
	fclose(fp);
}

// read tex file, and output transformed file
void do_read(FILE *fp, char title[])
{
	char buf[BUFSIZE*2], outputtitle[BUFSIZE], reaction[BUFSIZE];
	FILE *fileout;
	double parameters[12];
	int readposition,i,readitem,offset,checkitem,parameternum;

	// make out put file (***.tex -> ***.dat)
	strncpy(outputtitle, title, strlen(title)-3);
	outputtitle[strlen(title)-3] = '\0'; // string's last
	strcat(outputtitle,"dat");
	fprintf(stdout, "%s -> %s\n", title, outputtitle);
	fileout = fopen(outputtitle,"w");
	if (fileout == NULL) {
		printf("cannot open file(2)\n");
		return;
	}


	// read parameters
	readposition = OUT;
	while (1) {
		if (fgets(buf,sizeof(buf),fp) == NULL) 
			break;

		if (strncmp(buf,TOPRULE, strlen(TOPRULE))==0) {
			if (readposition == PARA) { printf("----total %2d\n", parameternum); }
			parameternum = 0;
			readposition = HEADER;
			continue;
		}
		if (strncmp(buf,BOTTOMRULE, strlen(BOTTOMRULE))==0) {
			readposition = OUT;
			printf("-----total %2d\n", parameternum);
			continue;
		}
		
		if (readposition == HEADER) {
			if (get_reaction(buf,reaction))
				readposition = HEADEREND;
			// plus alpha
			fgets(buf,sizeof(buf),fp);
			if (strncmp(buf,MIDRULE,strlen(MIDRULE)) == 0) {
				readposition = PARATITLE;
				continue;
			}
			comp_reaction(buf,reaction);
			continue;
		}
		if ((strncmp(buf,MIDRULE,strlen(MIDRULE)) == 0)) {
			if (readposition == HEADEREND) {
				readposition = PARATITLE;
			} else if (readposition = PARATITLE) {
				fprintf(stdout, "%s reaction\n", reaction);
				fprintf(fileout, "#%s\n", reaction);
				fprintf(fileout,
					"#run  title    deg   channel (err)  count  (err)    FC\n");
				readposition = PARA;
			}
			continue;
		}
		if (readposition == PARA) {
			offset = 0;
			if (buf[0] == '\r' || buf[0] == '\n' || buf[0] == '\\') // return
				continue;

			for (i = 0; i < 12; i++) {
				readitem = sscanf(&buf[offset],"%lf", &parameters[i]);
				if (readitem == 0) parameters[i] = ERR;
				offset = search_andposi(buf,offset);
			}
			// check 
			if ((parameters[3] == ERR) || (parameters[8] == ERR))
				continue;
			// output parameter 
			fprintf(fileout, "%04.0lf %s %4.1lf %8.1lf %5.1lf %8.1lf %5.1lf %8.0lf\n", 
					parameters[0], reaction, parameters[1], // run, deg
					parameters[3], parameters[4], // channel
					parameters[8], parameters[9], // Gross
					parameters[11]); // FC
			parameternum++;
		}
	}
	fclose(fileout);
}

int get_reaction(char buf[], char reaction[])
{
	int i,j,sw,count;
	char ast[] = "ast";

	j = 0;
	sw = 0;
	count = 0;
	for (i = strlen(HEADERTITLE); i < strlen(buf); i++) {
		if ((buf[i] >= '0' && buf[i] <= '9') || (buf[i] >= 'A' && buf[i] <= 'Z') || 
				(buf[i] >= 'a' && buf[i] <= 'z')) {
			if (buf[i] == 'Q') // Q-value part
				break;
			if (buf[i] == '\a') { // \\alpha
				reaction[j++] = 'a';
				reaction[j++] = 'l';
			} else {
				reaction[j++] = buf[i];
			}
			if (sw == 1) {
				sw = 2; // begin read *** excited state
			}
		}
		if (buf[i] == '$') { // set read *** excited state 
			sw = 1;
		}
		if (sw == 2 && buf[i] == ' ') // ***th 's end
			break;
	}
	reaction[j] = '\0';
	// remove ...^{\\ast}
	sw = 0; // search "ast"
	for (i = 0; i < strlen(reaction); i++) {
		if (sw == 0) {
			if (reaction[i] != 'a') 
				continue;
			if (reaction[i+1] != 's')
				continue;
			if (reaction[i+2] != 't')
				continue;
			sw = 1; // remove "ast"
			printf("need remove\n");
			i = i - 1;
		} else if (sw == 1) {
			reaction[i] = reaction[i+3];
		}
	}
	return TRUE;
}

// search and's position
int search_andposi(char buf[],int shift)
{
	int offset;
	for(offset = shift; offset < strlen(buf); offset++) {
		if (buf[offset] == '&')
			break;
	}
	offset++;
	return offset;
}

// for irregular tex file (not in one line reaction data)
void comp_reaction(char buf[], char reaction[])
{
	int length,i,j;

	length = strlen(reaction);
	j = 0;
	for (i = 0; i < strlen(buf); i++) {
		if ((buf[i] >= '1' && buf[i] <= '9') || 
				(buf[i] == 's') || (buf[i] == 't') ||
				(buf[i] == 'n') || (buf[i] == 'd') ||
				(buf[i] == 'h') || (buf[i] == 'r')) {
			reaction[length+j] = buf[i];
			j++;
		}
		if (buf[i] == 'Q')
			break;
	}
	reaction[length+j] = '\0';
}
